From bc13cbea5f7d2f546c31819a1431cb6680360e8d Mon Sep 17 00:00:00 2001
Message-Id: <bc13cbea5f7d2f546c31819a1431cb6680360e8d.1567406306.git.chenqiwu@xiaomi.com>
In-Reply-To: <cover.1567406306.git.chenqiwu@xiaomi.com>
References: <cover.1567406306.git.chenqiwu@xiaomi.com>
From: chenqiwu <chenqiwu@xiaomi.com>
Date: Mon, 2 Sep 2019 13:56:55 +0800
Subject: [PATCH 1/1] Fix for the determination of the ARM64 SECTION_SIZE_BITS.
 For some platforms may define hotplug section size bits in kernel config to
 support memory hotplug, which is not same as ARM64 default SECTION_SIZE_BITS.
 Without the patch, depending upon the hardware platform, the warning message
 maybe displayed:

crash> kmem -P 0x80000000
kmem: WARNING: cannot find mem_map page for address: fffffffe80000000
kmem: cannot determine page for 80000000
80000000: physical address not found in mem map

Signed-off-by: chenqiwu <chenqiwu@xiaomi.com>
---
 arm64.c | 33 ++++++++++++++++++++++++++++++++-
 1 file changed, 32 insertions(+), 1 deletion(-)

diff --git a/arm64.c b/arm64.c
index 6b34b5f..31c03d4 100644
--- a/arm64.c
+++ b/arm64.c
@@ -32,6 +32,7 @@ static int verify_kimage_voffset(void);
 static void arm64_calc_kimage_voffset(void);
 static void arm64_calc_phys_offset(void);
 static void arm64_calc_virtual_memory_ranges(void);
+static void arm64_get_section_size_bits(void);
 static int arm64_kdump_phys_base(ulong *);
 static ulong arm64_processor_speed(void);
 static void arm64_init_kernel_pgd(void);
@@ -375,7 +376,11 @@ arm64_init(int when)
 
 	case POST_GDB:
 		arm64_calc_virtual_memory_ranges();
-		machdep->section_size_bits = _SECTION_SIZE_BITS;
+		arm64_get_section_size_bits();
+		if (CRASHDEBUG(1)) {
+			fprintf(fp, "SECTION_SIZE_BITS: %ld\n", machdep->section_size_bits);
+		}
+
 		if (!machdep->max_physmem_bits) {
 			if ((string = pc->read_vmcoreinfo("NUMBER(MAX_PHYSMEM_BITS)"))) {
 				machdep->max_physmem_bits = atol(string);
@@ -1055,6 +1060,32 @@ arm64_calc_phys_offset(void)
 		fprintf(fp, "using %lx as phys_offset\n", ms->phys_offset);
 }
 
+/*
+ *  Determine SECTION_SIZE_BITS either by reading VMCOREINFO or the kernel
+ *  config, otherwise borrow the 64-bit ARM default definiton.
+ */
+static void
+arm64_get_section_size_bits(void)
+{
+	int ret;
+	char *string;
+
+	if ((string = pc->read_vmcoreinfo("NUMBER(SECTION_SIZE_BITS)"))) {
+		machdep->section_size_bits = atol(string);
+		free(string);
+		return;
+	}
+
+	if ((ret = get_kernel_config("CONFIG_MEMORY_HOTPLUG", NULL)) == IKCONFIG_Y) {
+		if ((ret = get_kernel_config("CONFIG_HOTPLUG_SIZE_BITS", &string)) == IKCONFIG_STR) {
+			machdep->section_size_bits = atol(string);
+			free(string);
+			return;
+		}
+	} 
+		
+	machdep->section_size_bits = _SECTION_SIZE_BITS;
+}
 
 /*
  *  Determine PHYS_OFFSET either by reading VMCOREINFO or the kernel
-- 
2.14.1

